/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/08 18:19:17
 */
import { html, css, Component, classMap, nextTick, outsideClick } from 'wkit'

import 'ui/icon/index.js'
import 'ui/scroll/index.js'
import 'ui/layer/index.js'
import 'ui/space/index.js'
import 'ui/form/input.js'
import 'ui/form/switch.js'
import 'ui/form/button.js'
import 'ui/form/link.js'

import './sidebar.js'
import './permission.js'
import './records.js'

import { checkPermission, getHistory, saveHosts, noop } from '../utils/index.js'

const HOST_DATA = await getHistory()

class Home extends Component {
  static props = {
    editDomain: '', // 当前临时要编辑的域名, 即右键菜单选择到的
    permissionShow: false
  }

  static styles = [
    css`
      :host {
        flex: 1;
        display: flex;
        width: 100%;
        height: 100%;
        padding: 32px;
        color: var(--color-dark-1);
        background: #f0f0f0;
      }
      .visible {
        display: block;
      }

      ul li {
        list-style: none;
      }

      .noselect {
        -webkit-touch-callout: none;
        -webkit-user-select: none;
        user-select: none;
      }
    `,

    css`
      .main {
        flex: 1;
        display: flex;
        flex-direction: column;
      }
      .main .toolbar {
        display: flex;
        align-items: center;
        justify-content: space-between;
        height: 40px;
        padding: 0 15px;
        border-bottom: 1px solid var(--color-plain-3);
        background: var(--color-plain-1);
      }
      .main .list {
        overflow: hidden;
        flex: 1;
      }
    `,

    css`
      .context-menu {
        display: flex;
        flex-direction: column;
        width: 100px;
        padding: 5px 0;
        background: #fff;
      }

      .context-menu .item {
        height: 30px;
        line-height: 30px;
        padding: 0 15px;
        cursor: pointer;
      }

      .context-menu .item :hover {
        background: #f2f5fc;
      }
    `
  ]

  async mounted() {
    window.foo = this

    let writable = await checkPermission()

    if (writable) {
      this.$store.HOST_DATA = HOST_DATA
      this.$store.domains = Object.keys(HOST_DATA)
      this.$refs.domain.mounted()
      outsideClick(this.$refs.context, _ => this.$refs.context.close())
    } else {
      this.permissionShow = true
    }
  }

  addRecord() {
    if (this.$store.activeDomain) {
      this.$store.records.push({
        record: '',
        value: '',
        enabled: true,
        remark: ''
      })
      nextTick(_ => (this.$refs.list.scrollTop = 1e6))
    } else {
      layer.toast('请先选择域名', 'warn')
    }
  }

  showMenu(ev) {
    this.$refs.context.close()
    var { pageX, pageY } = ev
    if (pageY + 70 > 600) {
      pageY -= 70
    }

    var elem = ev.target

    if (elem.tagName !== 'LI') {
      elem = elem.parentNode
    }
    this.editDomain = elem.dataset.name
    setTimeout(_ => {
      this.$refs.context.moveTo({ left: pageX + 'px', top: pageY + 'px' })
      this.$refs.context.show()
    })
  }

  confirmAction(ev) {
    this.$refs.context.close()
    if (ev.target.tagName === 'LI') {
      let act = ev.target.dataset.act
      let { HOST_DATA, records, domains } = this.$store
      let idx = domains.indexOf(this.editDomain)

      if (act === 'del') {
        layer
          .confirm(`是否要删除域名「${this.editDomain}」?`)
          .then(res => {
            if (this.editDomain === this.$store.activeDomain) {
              if (records.length) {
                return layer.toast(
                  '该域名下有主机记录, 请先删除主机记录后再删除域名',
                  'error'
                )
              }
            } else {
              if (HOST_DATA[this.editDomain].length > 0) {
                return layer.toast(
                  '该域名下有主机记录, 请先删除主机记录后再删除域名',
                  'error'
                )
              }
            }
            delete HOST_DATA[this.editDomain]

            domains.splice(idx, 1)

            this.editDomain = ''
            this.$store.records = []
            this.$store.activeDomain = domains[0]
            this.$refs.domain.mounted()
            this.save()
          })
          .catch(noop)
      } else if (act === 'edit') {
        layer
          .prompt(
            `请输入新的名字「${this.editDomain}」`,
            this.editDomain,
            (val, done) => {
              if (val === this.editDomain || HOST_DATA[val]) {
                return layer.toast(`${val} 域名没有变化, 或已经存在`)
              }

              if (
                val === 'localhost' ||
                val === 'local' ||
                /^[\w.]+\.[a-z]+$/.test(val)
              ) {
                done()
              } else {
                layer.toast('域名格式错误', 'error')
              }
            }
          )
          .then(val => {
            domains[idx] = val
            HOST_DATA[val] = HOST_DATA[this.editDomain]
            delete HOST_DATA[this.editDomain]

            this.$store.activeDomain = val
            this.editDomain = ''
            this.$refs.domain.mounted()
            this.save()
          })
          .catch(noop)
      }
    }
  }

  save() {
    let { HOST_DATA, activeDomain, records } = this.$store
    if (activeDomain) {
      HOST_DATA[activeDomain] = records.filter(it => it.record && it.value)
    }
    saveHosts(HOST_DATA)
    layer.toast('保存成功', 'success')
  }

  render() {
    return html`
      <wc-sidebar
        ref="domain"
        @toggle-domain=${_ => (this.$refs.list.scrollTop = 0)}
        @show-menu=${ev => this.showMenu(ev.event)}
        @save=${this.save}
      ></wc-sidebar>

      <main class="main noselect">
        <header class="toolbar">
          <wc-button size="m" icon="plus" @click=${this.addRecord}
            >新增记录</wc-button
          >
          <wc-button size="m" icon="fly" @click=${this.save}>保存</wc-button>
        </header>

        <wc-records class="list" ref="list"></wc-records>
      </main>

      <wc-permission
        class=${classMap({ visible: this.permissionShow })}
      ></wc-permission>

      <wc-layer ref="context" left="100px" top="0" radius="0">
        <ul class="context-menu" @click=${this.confirmAction}>
          <li class="item" data-act="del">删除域名</li>
          <li class="item" data-act="edit">修改域名</li>
        </ul>
      </wc-layer>
    `
  }
}

Home.reg('home')
